start.time  <-  Sys.time()

#########################
# LIBRARY AND FUNCTIONS #
#########################

source("/g/scb2/zaugg/carnold/Projects/AtacSeq/src/Snakemake/R/functions.R")

initFunctionsScript(packagesReq = NULL, minRVersion = "3.1.0", warningsLevel = 1, disableScientificNotation = TRUE)
checkAndLoadPackages(c("checkmate", "futile.logger", "tidyverse", "tools"), verbose = TRUE)


#########################
# ADDITIONAL PARAMETERS #
#########################

par.l = list()
par.l$log_minlevel = "INFO"
par.l$ggplot_binwidth = 1
par.l$ggplot_maxBreaks = 50
par.l$pdf_width = 12
par.l$pdf_height = 6
par.l$ggplot_axis.title.size = 16
par.l$ggplot_axis.text.x = 8
par.l$ggplot_axis.text.y = 12
par.l$ggplot_axis.col = "black"
par.l$ggplot_hist.col = "black"



############################################
# READ AND VALIDATE COMMAND LINE ARGUMENTS #
############################################

args <- commandArgs(trailingOnly = TRUE)

if (length(args) != 4) {
  stop("Expecting 4 arguments but found only  ", length(args), ". Exiting.")
} else {
 par.l$files_input      = args[1]
 par.l$cutoff           = as.numeric(args[2])
 par.l$file_outputPDF   = args[3]
 par.l$file_log         = args[4]
}


assertCharacter(par.l$files_input, len = 1)
assertIntegerish(par.l$cutoff, lower = 1)
assertDirectory(dirname(par.l$file_outputPDF), access = "w")
assertDirectory(dirname(par.l$file_log), access = "w")


startLogger(par.l$file_log, par.l$log_minlevel, appenderName = "file", removeOldLog = TRUE)
printParametersLog(par.l)



allFiles = strsplit(par.l$files_input, ",", fixed = TRUE)[[1]]



##################################
# Plot distribution of fragments #
##################################

pdf(par.l$file_outputPDF, width = par.l$pdf_width, height = par.l$pdf_height)


for (fileCur in allFiles) {
  
  assertFileExists(fileCur, access = "r")
  
  ##########################
  # READ AND PROCESS INPUT #
  ##########################

  tbl.df <- read_tsv(fileCur, col_names = FALSE, col_types = list(
      col_skip(), # "CHR,
      col_skip(), # "MAPQ"
      col_double() #"Read_length"
  ))
  
  if (nrow(problems(tbl.df)) > 0) {
      flog.fatal(paste0("Parsing errors: "), problems(tbl.df), capture = TRUE)
      stop("Error when parsing the file ", fileCur, ", see errors above")
  }
  
  tbl.df = tbl.df %>%
      dplyr::rename(Read_length = X3) %>%
      dplyr::filter(Read_length < par.l$cutoff, Read_length > 0) %>%
      mutate(Read_length = abs(Read_length))

  
  title = splitStringInMultipleLines(paste0(file_path_sans_ext(basename(fileCur))), 50)
 
  cap <- ggplot(tbl.df, aes(x = Read_length)) +
    geom_histogram(aes(y = ..count..),  color = par.l$ggplot_hist.col, binwidth = par.l$ggplot_binwidth) +
    xlab("Fragment length (in bp)") + ylab("Abundance") +
    ggtitle(title) + 
    scale_x_continuous(breaks = seq(0, par.l$cutoff, par.l$ggplot_maxBreaks), labels = seq(0,par.l$cutoff, par.l$ggplot_maxBreaks), limits = c(0, par.l$cutoff)) +
    theme(axis.text.x = element_text(color = par.l$ggplot_axis.col, size = par.l$ggplot_axis.text.x),
          axis.text.y = element_text(color = par.l$ggplot_axis.col, size = par.l$ggplot_axis.text.y),
          axis.title.x = element_text(colour = par.l$ggplot_axis.col, size = par.l$ggplot_axis.title.size, margin = margin(25,0,0,0)),
          axis.title.y = element_text(colour = par.l$ggplot_axis.col, size = par.l$ggplot_axis.title.size, margin = margin(0,25,0,0)),
          axis.line.x = element_line(color = par.l$ggplot_axis.col), axis.line.y = element_line(color = par.l$ggplot_axis.col),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          panel.border = element_blank(),
          panel.background = element_blank(),
          legend.position = c(0.1, 0.9),
          legend.justification = "center",
          legend.title = element_blank(),
          plot.title = element_text(hjust =  0.5)
          )
  
  print(cap)

  
}


dev.off()



.printExecutionTime(start.time)

flog.info("Session info: ", sessionInfo(), capture = TRUE)
