
#########################
# LIBRARY AND FUNCTIONS #
#########################

#source("/g/scb2/zaugg/zaugg_shared/scripts/Christian/src/R/functionsCollection.R")
source("/g/scb2/zaugg/carnold/Projects/AtacSeq/src/Snakemake/R/functions.R")



initFunctionsScript(packagesReq = NULL, minRVersion = "3.1.0", warningsLevel = 1, disableScientificNotation = TRUE)
checkAndLoadPackages(c("checkmate", "futile.logger", "tidyverse", "reshape2", "tools", "rlist", "scales", "grDevices", "gridExtra", "ggplot2", "Rsamtools", "GenomicRanges"), verbose = TRUE)
    

#########################
# ADDITIONAL PARAMETERS #
#########################

par.l = list()

# Currently hard-coded parameters
par.l$verbose = TRUE
par.l$commandSamtools = "samtools"
par.l$ggplot_axis.title.size = 20
par.l$ggplot_axis.text.x = 8
par.l$ggplot_axis.text.y = 12
par.l$ggplot_axis.col = "black"
par.l$ggplot.text.size = 8
par.l$nRowsPlotPDF = 2
par.l$log_minlevel = "INFO"


############################################
# READ AND VALIDATE COMMAND LINE ARGUMENTS #
############################################
args <- commandArgs(trailingOnly = TRUE)

if (length(args) != 12) {
    stop("Expecting 12 arguments but found only ", length(args),". Exiting.")
} else {
    par.l$sampleNames                    = strsplit(args[1], ",", fixed = TRUE)[[1]]
    par.l$rootDir                        = args[2]
    par.l$file_outputPDF                 = args[3]
    par.l$file_outputRData               = args[4]
    par.l$annotationFile                 = args[5]
    par.l$pairedEnd                      = as.logical(args[6])
    par.l$TSSEnrichment_withinThreshold  = as.integer(args[7])
    par.l$TSSEnrichment_outsideThreshold = as.integer(args[8])
    par.l$TSSEnrichment_geneTypesToKeep  = args[9]
    par.l$file_log                       = args[10]
    par.l$fileStatsPattern               = args[11]
    par.l$fileLibraryStatsPattern        = args[12]


}
# 
# par.l$sampleNames                    = strsplit("2FPCp15r1,2FPCp15r2,2FPCp25r1,2FPCp25r2,2FPCp5r1,2FPCp5r2,PCsr1,PCsr2,pOPCsr1,pOPCsr2", ",", fixed = TRUE)[[1]]        
# par.l$rootDir                        = "/scratch/bunina/atac/output"                 
# par.l$file_outputPDF                 = "/scratch/bunina/atac/output/Reports_and_Stats/sampleSummary/allSamples_statSummary.pdf "                
# par.l$file_outputRData               = "/scratch/bunina/atac/output/Reports_and_Stats/sampleSummary/allSamples_statSummary.RData"                 
# par.l$annotationFile                 = "/g/scb2/zaugg/zaugg_shared/annotations/mm10/Gencode_M16/gencode.vM16.annotation.gtf"                 
# par.l$pairedEnd                      = TRUE                 
# par.l$TSSEnrichment_withinThreshold  = 4000                 
# par.l$TSSEnrichment_outsideThreshold = 1000                  
# par.l$TSSEnrichment_geneTypesToKeep  ="protein_coding"                 
# par.l$file_log                       = "/scratch/bunina/atac/output/LOGS_AND_BENCHMARKS/stats.log"                
# par.l$fileStatsPattern               ="*.s.bam.stats$"
# par.l$fileLibraryStatsPattern        = "*.s.bam.statsLibraryCompl$"


startLogger(par.l$file_log, par.l$log_minlevel, appenderName = "file", removeOldLog = TRUE)

printParametersLog(par.l)

assertCharacter(par.l$sampleNames, min.len = 1, min.chars = 1)
assertDirectoryExists(dirname(par.l$file_outputPDF), access = "w")
assertLogical(par.l$pairedEnd)
assertDirectoryExists(dirname(par.l$file_log), access = "w")

assertIntegerish(par.l$TSSEnrichment_withinThreshold , lower = 0)
assertIntegerish(par.l$TSSEnrichment_outsideThreshold , lower = 0)
assertCharacter(par.l$TSSEnrichment_geneTypesToKeep , len = 1, min.chars = 1)

assertDirectoryExists(par.l$rootDir, access = "r")

assertFileExists(par.l$annotationFile, access = "r")
assertCharacter(par.l$fileStatsPattern, len = 1, min.chars = 1)
assertCharacter(par.l$fileLibraryStatsPattern, len = 1, min.chars = 1)
# Check existence of output directories

dir_beforeTrimming = paste0(par.l$rootDir, "/1.FastQC_beforeTrimming")
assertDirectoryExists(dir_beforeTrimming, access = "r")

dir_afterTrimming = paste0(par.l$rootDir, "/3.FastQC_afterTrimming")
assertDirectoryExists(dir_afterTrimming, access = "r")

dir_alignment = paste0(par.l$rootDir, "/4.Alignment")
assertDirectoryExists(dir_alignment, access = "r")


dir_rmChrm = paste0(par.l$rootDir, "/5.Postalignment/3.Filter_chrM")
assertDirectoryExists(dir_rmChrm, access = "r")

dir_rmDUP = paste0(par.l$rootDir, "/5.Postalignment/4.MarkAndRemove_Duplicates")
assertDirectoryExists(dir_rmDUP, access = "r")

dir_MAPQ = paste0(par.l$rootDir, "/5.Postalignment/5.Filter_MAPQ")
assertDirectoryExists(dir_MAPQ, access = "r")

dir_FINAL = paste0(par.l$rootDir, "/8.FinalOutput")
assertDirectoryExists(dir_FINAL, access = "r")

output.l = list()

######################
# Search stats files #
######################

stats.df = tribble(~sampleName, ~BT, ~AT, ~align, ~alignrate, ~MAPQsort, ~chrM, ~rmDUP, 
                      ~readsTotal , ~readsDistinct, ~readsOccOnce, ~readsOccTwo , ~NRF, ~PBC1, ~PBC2)


for (sampleNameCur in par.l$sampleNames) {
    
    trimmingData = c()
    
      for (dirCur in c(dir_beforeTrimming, dir_afterTrimming)) {
          
          # 1. FASTQC
          files = createFileList(dir_beforeTrimming, "fastqc_data.txt", verbose = FALSE, recursive = TRUE)
          # So far contains all samples, grep for the current one
          files = files[which(grepl(paste0("/", sampleNameCur,".*_fastqc"), files))]
          stopifnot(length(files) == 2)
          command = paste0("less ", dir_beforeTrimming, "/", sampleNameCur, "*_fastqc/fastqc_data.txt")
          stat = system(command, intern = TRUE)
          #stat = system2(command)
          numCur = as.numeric(unlist(strsplit(stat[7], "\t", fixed = TRUE))[2])
          if (par.l$pairedEnd) numCur = numCur *2
          trimmingData = c(trimmingData, numCur)
      }


      # 3. Alignment
      #Ivan bug remove * from .s.bam$
	    file = createFileList(dir_alignment, paste0(sampleNameCur, ".s.bam$"), verbose = FALSE)
	    file = file[which(grepl(paste0("/", sampleNameCur,".s.bam"), file))]
      stopifnot(length(file) == 1)
      command = paste0(par.l$commandSamtools, " flagstat ", file)
      align_stat = system(command, intern = TRUE)
      align_num  = as.numeric(unlist(strsplit(align_stat[5],"+", fixed = TRUE))[1])
      align_rate = as.numeric(unlist(strsplit(align_stat[5],"[% | (]", fixed = FALSE))[6])
      
      # 4. Postalign chrM
	  # Ivan add "." syntax to search
      file = createFileList(dir_rmChrm, paste0(sampleNameCur,".", par.l$fileStatsPattern), verbose = FALSE)
      file = file[which(grepl(paste0("/", sampleNameCur,".", par.l$fileStatsPattern), file))]
      stopifnot(length(file) == 1)
      command = paste0("less ", file)
      chrM_stat = system(command, intern = TRUE)
      chrm_num = as.numeric(unlist(strsplit(chrM_stat[1], "+", fixed = TRUE))[1])
      
      # 5. Postalign rmDUP
	  # ths same as above
      file = createFileList(dir_rmDUP, paste0(sampleNameCur,".", par.l$fileStatsPattern), verbose = FALSE)
      file = file[which(grepl(paste0("/", sampleNameCur,".", par.l$fileStatsPattern), file))]
      stopifnot(length(file) == 1)
      command = paste0("less ", file)
      rmDup_stat = system(command, intern = TRUE)
      rmDup_num = as.numeric(unlist(strsplit(rmDup_stat[1],"+", fixed = TRUE))[1])
    
      # 6. Postalign MAPQ
	  # the same as above
      file = createFileList(dir_MAPQ, paste0(sampleNameCur,".", par.l$fileStatsPattern), verbose = FALSE)
      file = file[which(grepl(paste0("/", sampleNameCur,".", par.l$fileStatsPattern), file))]
      stopifnot(length(file) == 1)
      command = paste0("less ", file)
      MAPQsort_stat = system(command, intern = TRUE)
      MAPQsort_num = as.numeric(unlist(strsplit(MAPQsort_stat[1],"+", fixed = TRUE))[1])
      
      # 7. Postalign rmINDEL
	  # the same as above
      #file = createFileList(dir_INDEL, paste0(sampleNameCur,".", par.l$fileStatsPattern), verbose = FALSE)
      #stopifnot(length(file) == 1)
      #command = paste0("less ", file)
      #rmINDEL_stat = system(command, intern = TRUE)
      #rmINDEL_num = as.numeric(unlist(strsplit(rmINDEL_stat[1],"+", fixed = TRUE))[1])
      
      # Library complexity numbers
	  # the same as above 
      file = createFileList(dir_rmDUP, paste0(sampleNameCur,".", par.l$fileLibraryStatsPattern), verbose = FALSE)
      file = file[which(grepl(paste0("/", sampleNameCur,".", par.l$fileLibraryStatsPattern), file))]
      stopifnot(length(file) == 1)
      libraryCompl.df = read.table(file, header = TRUE, stringsAsFactors = FALSE, sep = "\t")
      stopifnot(nrow(libraryCompl.df) == 1)


      stats.df = add_row(stats.df, 
                         sampleName = sampleNameCur,
                         BT = trimmingData[1], 
                         AT = trimmingData[2],
                         align = align_num, 
                         alignrate = align_rate,
                         MAPQsort = MAPQsort_num, 
                         chrM = chrm_num,
                         rmDUP = rmDup_num, 
                        # rmINDEL = rmINDEL_num,
                         readsTotal    = libraryCompl.df$readsTotal,
                         readsDistinct = libraryCompl.df$readsDistinct,
                         readsOccOnce  = libraryCompl.df$readsOccOne,
                         readsOccTwo   = libraryCompl.df$readsOccTwo,
                         NRF           = libraryCompl.df$NRF,
                         PBC1          = libraryCompl.df$PBC1,
                         PBC2          = libraryCompl.df$PBC2
                         )
}


##################
# TSS enrichment #
##################

finalOutputFiles = paste0(dir_FINAL, "/", stats.df$sampleName, ".final.bam")

res.l = list()

for (fileCur in finalOutputFiles) {

    assertDirectory(dirname(fileCur), access = "w")
        
    res.l[[fileCur]] = checkEnrichmentOverTSS(par.l$annotationFile, 
                                              fileCur, 
                                              thresholdWithinTSS = par.l$TSSEnrichment_withinThreshold,
                                              thresholdOutsideTSS = par.l$TSSEnrichment_outsideThreshold,
                                              readFlags = generateDefaultReadFlags(pairedEndReads = par.l$pairedEnd),
                                              geneTypesToKeep = par.l$TSSEnrichment_geneTypesToKeep, 
                                              verbose = par.l$verbose)

}

########
# PLOT #
########

enrichmentRaw.l = list.select(res.l, overallEnrichmentRaw)


# reshape to plot graphs

qualityMetrics.df <- data.frame(sampleName    = stats.df$sampleName,
                          nReadsMin     = stats.df$MAPQsort,
                          nReadsMax     = stats.df$BT,
                          NRF           = stats.df$NRF,
                          PBC1          = stats.df$PBC1,
                          PBC2          = stats.df$PBC2,
                          enrichmentRaw = NA)

qualityMetrics.df = mutate(qualityMetrics.df, 
                     NRF_color  = ifelse(NRF < 0.7 , "orange", ifelse(NRF > 0.9 , "forestgreen", "yellow")), 
                     PBC1_color = ifelse(PBC1 < 0.7, "orange", ifelse(PBC1 > 0.9, "forestgreen", "yellow")),
                     PBC2_color = ifelse(PBC2 < 1  , "orange", ifelse(PBC2 > 3  , "forestgreen", "yellow"))
          )


qualityMetrics.df$NRF_colorFac = factor(qualityMetrics.df$NRF_color  , levels = c("forestgreen","yellow", "orange"))
qualityMetrics.df$PBC1_colorFac = factor(qualityMetrics.df$PBC1_color, levels = c("forestgreen","yellow", "orange"))
qualityMetrics.df$PBC2_colorFac = factor(qualityMetrics.df$PBC2_color, levels = c("forestgreen","yellow", "orange"))

# IVAN: this was super annoying bug add regex to find only with fixed specific ending
for (i in seq_len(nrow(qualityMetrics.df))) {
  cleanName = gsub(".final.bam", "", basename(names(enrichmentRaw.l)[i]), fixed = TRUE)
  index = which(qualityMetrics.df$sampleName == cleanName)
  stopifnot(length(index) == 1)
  qualityMetrics.df$enrichmentRaw[index] = enrichmentRaw.l[[i]]$overallEnrichmentRaw
}

stopifnot(!any(is.na(qualityMetrics.df$enrichmentRaw)))

# Add Percentages
stats.df = mutate(stats.df, 
                    BT_perc       = BT       * 100 / BT,
                    AT_perc       = AT       * 100 / BT,
                    align_perc    = align    * 100 / BT,
                    chrM_perc     = chrM     * 100 / BT,
                    rmDUP_perc    = rmDUP    * 100 / BT,
                    MAPQsort_perc = MAPQsort * 100 / BT
                  )



#colnames(stats.df.perc) = c("sampleName","Before\ntrimming","After\ntrimming","Alignment","MAPQ\nfilter","Filter\nchrM"," Filter\nduplicates"," Filter\nINDELs")

stats.subset.df  = dplyr::select(stats.df, one_of("sampleName", "BT_perc", "AT_perc", "align_perc", "chrM_perc", "rmDUP_perc", "MAPQsort_perc"))
stats.subset2.df = dplyr::select(stats.df, one_of("sampleName", "BT", "AT", "align", "chrM", "rmDUP", "MAPQsort"))

colnames(stats.subset.df) = colnames(stats.subset2.df) = c("sampleName","Before\ntrimming","After\ntrimming","Alignment","Filter\nchrM", "Filter\nduplicates", "MAPQ\nfilter")



stats.red.melted.df = melt(stats.subset.df)
stats.red.melted2.df = melt(stats.subset2.df)



#pdf_width = (nrow(stats.df) / par.l$nRowsPlotPDF) * 7 # 6 width per column
pdf_width = 10
pdf_height = 10 # fixed because of always two rows


pdf(par.l$file_outputPDF, width = pdf_width, height = pdf_height)


pos_x = 5.7

# stats.plot = ggplot(stats.red.melted.df, aes(x = variable, y = value)) + geom_bar(stat = "identity", fill = "palevioletred4", color = "whitesmoke", size = 1.5) +
#   facet_wrap(~sampleName, nrow = par.l$nRowsPlotPDF) + xlab("Filtering steps in pipeline") + ylab("% of the initial number of reads") +
#   geom_label(aes(x = variable, y = value, label = paste0(round(value,1),"%") , vjust = 0.5, hjust = 0.5), size = 4, color = "darkblue") +
#   theme(axis.text.x = element_text(face = "bold", color = par.l$ggplot_axis.col, size = par.l$ggplot_axis.text.x),
#         axis.text.y = element_text(face = "bold", color = par.l$ggplot_axis.col, size = par.l$ggplot_axis.text.y),
#         axis.title.x = element_text(face = "bold", colour = par.l$ggplot_axis.col, size = par.l$ggplot_axis.title.size, margin = margin(25,0,0,0)),
#         axis.title.y = element_text(face = "bold", colour = par.l$ggplot_axis.col, size = par.l$ggplot_axis.title.size, margin = margin(0,25,0,0)),
#         axis.line.x = element_line(color = par.l$ggplot_axis.col), axis.line.y = element_line(color = "black"),
#         panel.grid.major = element_blank(),
#         panel.grid.minor = element_blank(),
#         panel.border = element_blank(),
#         panel.background = element_blank(),
#         legend.position = c(0.1, 0.9),
#         legend.justification = "center",
#         legend.title = element_blank())  +
#   theme(axis.text.x = element_text(angle = 90, hjust = 0.5, vjust = 0.5)) + 
#   geom_text(data = qualityMetrics.df, aes(x = pos_x, y = 100, label = paste("Start: ", .prettyNum(nReadsMax))), size = 3, hjust = 0) +
#   geom_text(data = qualityMetrics.df, aes(x = pos_x, y = 95, label = paste("Final: ", .prettyNum(nReadsMin))), size = 3, hjust = 0) +
#   geom_text(data = qualityMetrics.df, aes(x = pos_x, y = 90, label = paste("TSS:", enrichmentRaw, ", NRF:", round(NRF,2))), size = 3, hjust = 0) +
#   geom_text(data = qualityMetrics.df, aes(x = pos_x, y = 85, label = paste("PBC1/2:", round(PBC1,2), "/", round(PBC2,2))), size = 3, hjust = 0)

# print(stats.plot)
allSamples = unique(stats.df$sampleName)

statsSamples.l = list()

for (sampleCur in allSamples) {
  local({
    subsetSample.df = filter(stats.subset.df, sampleName %in% sampleCur)
    stopifnot(nrow(subsetSample.df) == 1)
    subsetSample.melted.df = melt(subsetSample.df)
    
    qualityMetrics.filt.df = filter(qualityMetrics.df, sampleName %in% sampleCur)
    
    stats.plot = ggplot(subsetSample.melted.df, aes(x = variable, y = value)) + geom_bar(stat = "identity", fill = "palevioletred4", color = "whitesmoke", size = 1.5) +
      xlab("Filtering steps in pipeline") + ylab("% of the initial number of reads") +
      geom_label(aes(x = variable, y = value, label = paste0(round(value,1),"%") , vjust = 0.5, hjust = 0.5), size = 4, color = "darkblue") +
      theme(axis.text.x = element_text(face = "bold", color = par.l$ggplot_axis.col, size = par.l$ggplot_axis.text.x),
            axis.text.y = element_text(face = "bold", color = par.l$ggplot_axis.col, size = par.l$ggplot_axis.text.y),
            axis.title.x = element_text(face = "bold", colour = par.l$ggplot_axis.col, size = par.l$ggplot_axis.title.size, margin = margin(25,0,0,0)),
            axis.title.y = element_text(face = "bold", colour = par.l$ggplot_axis.col, size = par.l$ggplot_axis.title.size, margin = margin(0,25,0,0)),
            axis.line.x = element_line(color = par.l$ggplot_axis.col), axis.line.y = element_line(color = "black"),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border = element_blank(),
            panel.background = element_blank(),
            legend.position = c(0.1, 0.9),
            legend.justification = "center",
            legend.title = element_blank())  +
      theme(axis.text.x = element_text(angle = 90, hjust = 0.5, vjust = 0.5)) + 
      ggtitle(basename(sampleCur)) + 
      geom_text(data = qualityMetrics.filt.df, aes(x = pos_x, y = 100, label = paste("Start: ", .prettyNum(nReadsMax))), size = 3, hjust = 0) +
      geom_text(data = qualityMetrics.filt.df, aes(x = pos_x, y = 95, label = paste("Final: ", .prettyNum(nReadsMin))), size = 3, hjust = 0) +
      geom_text(data = qualityMetrics.filt.df, aes(x = pos_x, y = 90, label = paste("TSS:", enrichmentRaw, ", NRF:", round(NRF,2))), size = 3, hjust = 0) +
      geom_text(data = qualityMetrics.filt.df, aes(x = pos_x, y = 85, label = paste("PBC1/2:", round(PBC1,2), "/", round(PBC2,2))), size = 3, hjust = 0)
    
    print(stats.plot)
    statsSamples.l[[sampleCur]] <<- stats.plot
  })
    
}


p2 <- ggplot(qualityMetrics.df)
p2 <- p2 + aes(x = sampleName, y = NRF, fill = NRF_colorFac)
p2 <- p2 + geom_bar(stat = 'identity')
p2 <- p2 + ylim(0, 1)
p2 <- p2 + .getThemeForGGPlot()
p2 <- p2 + ggtitle("Non-Redundant Fraction (NRF): \nNo. distinct uniquely mapping reads / No. reads total")
p2 <- p2 + geom_hline(yintercept = 0.9, color = "yellow", size = 1, alpha = 0.5)
p2 <- p2 + geom_hline(yintercept = 0.7, color = "orange", size = 1, alpha = 0.5)
p2 <- p2 + theme(axis.text.x = element_text(angle = 90, hjust = 1, size = 5))
p2 <- p2 + scale_fill_manual(name = "ENCODE\nlibrary\ncomplexity", values = c("forestgreen" = "forestgreen", "yellow" = "yellow", "orange" = "orange"), labels = c("forestgreen" = "Ideal", "yellow" = "Acceptable", "orange" = "Concerning"))



p3 <- ggplot(qualityMetrics.df)
p3 <- p3 + aes(x = sampleName, y = PBC1, fill = PBC1_colorFac)
p3 <- p3 + geom_bar(stat = 'identity')
p3 <- p3 + ylim(0, 1)
p3 <- p3 + .getThemeForGGPlot()
p3 <- p3 + ggtitle("Library Complexity:\nPCR Bottlenecking Coefficient 1 (PBC1)")
p3 <- p3 + geom_hline(yintercept = 0.9, color = "yellow",  size = 1, alpha = 0.5)
p3 <- p3 + geom_hline(yintercept = 0.7, color = "orange", size = 1, alpha = 0.5)
p3 <- p3 + theme(axis.text.x = element_text(angle = 90, hjust = 1, size = 5))
p3 <- p3 + scale_fill_manual(name = "ENCODE\nbottlenecking\nlevel", values = c("forestgreen" = "forestgreen", "yellow" = "yellow", "orange" = "orange"), labels = c("forestgreen" = "None", "yellow" = "Moderate", "orange" = "Severe"))

p4 <- ggplot(qualityMetrics.df)
p4 <- p4 + aes(x = sampleName, y = PBC2, fill = PBC2_colorFac)
p4 <- p4 + geom_bar(stat = 'identity')
p4 <- p4 + ylim(0, NA)
p4 <- p4 + .getThemeForGGPlot()
p4 <- p4 + ggtitle("Library Complexity:\nPCR Bottlenecking Coefficient 2 (PBC2)")
p4 <- p4 + geom_hline(yintercept = 3, color = "yellow", size = 1, alpha = 0.5)
p4 <- p4 + geom_hline(yintercept = 1, color = "orange", size = 1, alpha = 0.5)
p4 <- p4 + theme(axis.text.x = element_text(angle = 90, hjust = 1, size = 5))
p4 <- p4 + scale_fill_manual(name = "ENCODE\nbottlenecking\nlevel", values = c("forestgreen" = "forestgreen", "yellow" = "yellow", "orange" = "orange"), labels = c("forestgreen" = "None", "yellow" = "Moderate", "orange" = "Severe"))



p5 <- ggplot(stats.red.melted.df, aes(x = variable, y = value))
p5 <- p5 + geom_boxplot() + geom_jitter(height = 0, alpha = 0.1, color = "black")
p5 <- p5 + .getThemeForGGPlot()
p5 <- p5 + ggtitle("Summary overview")
p5 <- p5 + ylab("% of the initial number of reads")
p5 <- p5 + xlab("Filtering steps in pipeline")

p6 <- ggplot(stats.red.melted2.df, aes(x = variable, y = value))
p6 <- p6 + geom_boxplot(outlier.shape = NA) + geom_jitter(height = 0, alpha = 0.3, color = "black")
p6 <- p6 + .getThemeForGGPlot()
p6 <- p6 + ggtitle("Summary overview")
p6 <- p6 + scale_y_continuous(name = "Number of reads", labels = comma)
p6 <- p6 + xlab("Filtering steps in pipeline")



print(p5)
print(p6)
printMultipleGraphsPerPage(list(p2,p3,p4), nCol = 1, nRow = 3, pdfFile = NULL, height = pdf_height, width = pdf_width)
# print(p2)
# print(p3)
# print(p4)
dev.off()

# Write all data to an rds file
output.l$summaryStatistics = stats.df
output.l$TSSEnrichment     = res.l
output.l$qualityMetrics    = qualityMetrics.df
output.l$plots             = list(summary = statsSamples.l, NRF = p2, PBC1 = p3, PBC2 = p4, overview1 = p5, overview2 = p6)

    
saveRDS(output.l, par.l$file_outputRData)

flog.info(paste0("\nGenerated file", par.l$file_outputPDF, " and ", par.l$file_outputRData, " and finished script successfully.\n"))
