
#########################
# LIBRARY AND FUNCTIONS #
#########################

#source("/g/scb2/zaugg/zaugg_shared/scripts/Christian/src/R/functionsCollection.R")
source("/g/scb2/zaugg/carnold/Projects/AtacSeq/src/Snakemake/src/functions.R")





start.time  <-  Sys.time()


#########################
# LIBRARY AND FUNCTIONS #
#########################

library("checkmate")
assertClass(snakemake, "Snakemake")
assertDirectoryExists(snakemake@config$par_general$dir_scripts)
source(paste0(snakemake@config$par_general$dir_scripts, "/functions.R"))

initFunctionsScript(packagesReq = NULL, minRVersion = "3.1.0", warningsLevel = 1, disableScientificNotation = TRUE)
checkAndLoadPackages(c("checkmate", "futile.logger", "tidyverse", "reshape2", "tools", "rlist", "scales", "grDevices", "gridExtra", "ggplot2", "Rsamtools", "GenomicRanges"), verbose = TRUE)

########################################################################
# SAVE SNAKEMAKE S4 OBJECT THAT IS PASSED ALONG FOR DEBUGGING PURPOSES #
########################################################################

# snakemake = readRDS("/scratch/carnold/CLL/27ac_TF/output/Logs_and_Benchmarks/3.analyzeTF.R_TF=MAFK.S.rds")
createDebugFile(snakemake, "3.analyzeTF.R")

###################
#### PARAMETERS ###
###################

par.l = list()

par.l$verbose = TRUE
par.l$log_minlevel = "INFO"
par.l$maxPairwiseComparisonsDiagnosticPermutations = 2

#####################
# VERIFY PARAMETERS #
#####################

assertClass(snakemake, "Snakemake")

## INPUT ##
assertList(snakemake@input, min.len = 1)
assertSubset(names(snakemake@input), c("", "overlapFile", "sampleDataR", "peakFile", "peakFile2", "normFacs", "plotsPerm"))

par.l$file_input_peakTFOverlaps  = snakemake@input$overlapFile
assertFileExists(par.l$file_input_peakTFOverlaps, access = "r")


## OUTPUT ##
assertList(snakemake@output, min.len = 1)
assertSubset(names(snakemake@output), c("", "outputTSV", "outputRDS", "plot_diagnostic", "plot_diagnosticPerm", "plot_TFSummary", "plot_TFSummaryPerm", "DESeqObj"))

par.l$file_output_summaryAll      = snakemake@output$outputTSV


## WILDCARDS ##
assertList(snakemake@wildcards, min.len = 1)
assertSubset(names(snakemake@wildcards), c("", "TF"))

par.l$TF = snakemake@wildcards$TF
assertCharacter(par.l$TF, len = 1, min.chars = 1)

## CONFIG ##
assertList(snakemake@config, min.len = 1)

par.l$designFormula = snakemake@config$par_general$designContrast
assertCharacter(par.l$designFormula, len = 1, min.chars = 3)

par.l$designFormula = snakemake@config$par_general$designContrast
checkAndLogWarningsAndErrors(par.l$designFormula, checkCharacter(par.l$designFormula, len = 1, min.chars = 3))

## PARAMS ##
assertList(snakemake@params, min.len = 1)
assertSubset(names(snakemake@params), c("", "doCyclicLoess", "allBAMS"))

## LOG ##
assertList(snakemake@log, min.len = 1)
par.l$file_log = snakemake@log[[1]]


allDirs = c(dirname(par.l$file_output_summaryAll), 
            dirname(par.l$file_log)
)


testExistanceAndCreateDirectoriesRecursively(allDirs)


######################
# FINAL PREPARATIONS #
######################
startLogger(par.l$file_log, par.l$log_minlevel, appenderName = "file", removeOldLog = TRUE)
printParametersLog(par.l)